script.on_event(defines.events.on_gui_opened, function(event)
    local entity = event.entity
    if entity and entity.name == 'parm-combinator' then
        -- close this gui and open a lua one, or attach to player.gui.relative
    end
end)

local function on_created_entity(event)
    local entity = event.created_entity or event.entity or event.destination

    entity.active = false

    local internal_combinator = entity.surface.find_entity('parm-combinator-internal', entity.position)
    if internal_combinator == nil then
        internal_combinator =
            entity.surface.create_entity { name = 'parm-combinator-internal', force = entity.force, position = entity.position }

        internal_combinator.destructible = false

        internal_combinator.connect_neighbour({
            target_entity = entity,
            wire = defines.wire_type.red,
            target_circuit_id = defines.circuit_connector_id.combinator_output,
        })
        internal_combinator.connect_neighbour({
            target_entity = entity,
            wire = defines.wire_type.green,
            target_circuit_id = defines.circuit_connector_id.combinator_output,
        })

        global.children[script.register_on_entity_destroyed(entity)] = internal_combinator
    end

    global.structs[entity.unit_number]
    = { unit_number = entity.unit_number, entity = entity, internal = internal_combinator, values = {}, rates = {}, next_tick = 0 }
end

for _, event in ipairs({
    defines.events.on_built_entity,
    defines.events.on_robot_built_entity,
    defines.events.script_raised_built,
    defines.events.script_raised_revive,
    defines.events.on_entity_cloned,
}) do
    script.on_event(event, on_created_entity, { { filter = 'name', name = 'parm-combinator' }, })
end

local function on_configuration_changed()
    global.structs = global.structs or {}
    global.children = global.children or {}
end

script.on_init(on_configuration_changed)
script.on_configuration_changed(on_configuration_changed)

script.on_nth_tick(60, function(event)
    for unit_number, struct in pairs(global.structs) do
        local signal_port = defines.circuit_connector_id.combinator_input
        local red_network = struct.entity.get_circuit_network(defines.wire_type.red, signal_port)

        local estimate = 0 / 0
        log("estimate: ".. estimate)
        local out = {}
        table.insert(out, { signal = red_network.signals[1].signal, count = estimate, index = #out + 1 })
        struct.internal.get_control_behavior().parameters = out
    end
end)

script.on_event(defines.events.on_entity_destroyed, function(event)
    local child = global.children[event.registration_number]
    if child then
        global.children[event.registration_number] = nil
        child.destroy()
    end
end)
